package com.sample;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.drools.KnowledgeBase;
import org.drools.builder.ResourceType;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.jbpm.process.workitem.wsht.SyncWSHumanTaskHandler;
import org.jbpm.task.TaskService;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.local.LocalTaskService;
import org.jbpm.test.JbpmJUnitTestCase;
import org.junit.Test;

/**
 * This is a sample file to launch a process.
 */
public class ProcessMultiThreadTest extends JbpmJUnitTestCase {

    private static int THREAD_NUM = 2;

    public ProcessMultiThreadTest() {
        super(true);
        setPersistence(true);
    }

    @Test
    public void testProcess() {

        Map<String, ResourceType> kbtypes = new HashMap<String, ResourceType>();
        kbtypes.put("BPMN2-RuleTask.drl", ResourceType.DRL);
        kbtypes.put("BPMN2-RuleTask.bpmn2", ResourceType.BPMN2);

        KnowledgeBase kbase = null;
        try {
            kbase = createKnowledgeBase(kbtypes);
        } catch (Exception e) {
            e.printStackTrace();
        }

        ExecutorService executor = Executors.newFixedThreadPool(THREAD_NUM);

        RuleTaskJob[] RuleTaskJobs = new RuleTaskJob[THREAD_NUM];
        for (int i = 0; i < THREAD_NUM; i++) {
            StatefulKnowledgeSession ksession = createKnowledgeSession(kbase);
            
            RuleTaskJobs[i] = new RuleTaskJob(ksession);
        }

        for (int i = 0; i < THREAD_NUM; i++) {
            executor.execute(RuleTaskJobs[i]);
        }
        
        try {
            Thread.sleep(10000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        System.out.println("main end");
    }

    private static class RuleTaskJob implements Runnable {

        private StatefulKnowledgeSession ksession;

        public RuleTaskJob(StatefulKnowledgeSession ksession) {
            this.ksession = ksession;
        }

        @Override
        public void run() {
            List<String> list = new ArrayList<String>();
            ksession.setGlobal("list", list);
            ProcessInstance processInstance = ksession.startProcess("RuleTask");
            long pid =  processInstance.getId();
            
            System.out.println("processInstance started : pid = " + pid);

            ksession.fireAllRules();
            
            System.out.println("fired : pid = " + pid);

            ksession.dispose();
        }

    }
}